// Tractrix curve by Brian McMillin, May 9, 2017.
// https://en.wikipedia.org/wiki/Tractrix
// http://mathworld.wolfram.com/Tractrix.html

//	Define hyperbolic trigonometry functions.
function cosh(x)
{
	return (Math.exp(x) + Math.exp(-x))/2.0;
}

function sech(x)
{
	return 2/(Math.exp(x) + Math.exp(-x));
}

function sinh(x)
{
	return (Math.exp(x) - Math.exp(-x))/2.0;
}

function tanh(x)
{
	return (Math.exp(x) - Math.exp(-x))/(Math.exp(x) + Math.exp(-x));
}

function EvalTractrix( t, maxWidth, aParam )
{
	// Calculate Tractrix from x = 0 to x = +/- maxWidth.
	var t = t * ((maxWidth / aParam) +1);
	
	// Tractrix formula.
	var x = aParam * (t - tanh(t));
	var y = aParam * (sech(t)); 
	var z = 0.0;
	
	return moi.vectorMath.createPoint( x, y, z );
}

function Update()
{
	var factory = moi.command.createFactory( 'interpcurve' );

	var maxWidth = moi.ui.commandUI.maxWidth.numericValue;
	var aParam = moi.ui.commandUI.aParam.numericValue;
	var numpoints = moi.ui.commandUI.numpoints.integerValue;
	numpoints += 1;
	var leftCurve = moi.ui.commandUI.leftCurve.value;
	var leftSide = 1;
	if (leftCurve)
		leftSide = -1;	
	
	for ( var i = 0; i < numpoints; ++i )
	{
		var t = leftSide * (i / (numpoints - 1));		
		var pt = EvalTractrix( t, maxWidth, aParam );		
		
		factory.createInput( 'point' );
		factory.setInput( factory.numInputs - 1, pt );
	}
	
	factory.update();
	
	return factory;
}

function DoTractrix()
{
	var factory = Update();

	var dlg = moi.ui.commandDialog;
	
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			factory.cancel();
			return false;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'maxWidth' || dlg.event == 'aParam' || dlg.event == 'numpoints' || dlg.event == 'leftCurve' )
		{
			factory.cancel();
			factory = Update();
		}
	}
	
	factory.commit();
}

DoTractrix();
